import Vue from 'vue'
import VueRouter from 'vue-router'
import util from 'assets/js/util.js'
import _ from 'underscore'

const historyList = []

import Index from 'components/index/Index.vue'

Vue.use(VueRouter)

const routes = [
  {path: '/', component: Index}
]

const router = new VueRouter({
  routes
})

function buildLocation (location) {
  return {
    location: location
  }
}

function isLocationEqual (location1, location2) {
  if (location1.path !== location2.path || !util.isEqual(location1.query, location2.query)) {
    return false
  }
  return true
}

router.oldReplace = router.replace
router.replace = function (location) {
  historyList.pop()
  historyList.push(buildLocation(location))
  return this.oldReplace(location)
}

router.oldPush = router.push
router.push = function (location) {
  historyList.push(buildLocation(location))
  return this.oldPush(location)
}

router.beforeEach((to, from, next) => {
  var length = historyList.length
  // 打开首页时
  if (length === 0) {
    historyList.push(buildLocation(to))
    next()
    return
  }

  var backStep = 0
  var isHistoryBack = false

  // 判断最后一条历史是否跟目标一致，如不一致则代表用户后退了页面
  var last = _.last(historyList)
  if (!isLocationEqual(last.location, to)) {
    isHistoryBack = true
  }
  // 遍历历史记录，跳过最后一条
  for (var i = 0; i < length - 1; i++) {
    // 如果当前页面在历史记录中，在记录中添加当前的页面滚动位置
    if (isLocationEqual(historyList[i].location, from)) {
      var top = window.pageYOffset || document.documentElement.scrollTop
      historyList[i].top = top
    }
    //  如果目标页面在历史记录中，删掉目标页面之后的历史记录，并计算目标页面距当前页面的步数
    if (isLocationEqual(historyList[i].location, to)) {
      backStep = i - length + 2
      for (var j = 0; j < -backStep + 1; j++) {
        historyList.pop()
      }
      break
    }
  }
  if (!isHistoryBack && backStep !== 0) {
    router.go(backStep)
  } else {
    next()
  }
})

Vue.directive('keep-scroll-position', {
  inserted: function (el) {
    var location = _.last(historyList)
    if (location.top) {
      document.body.scrollTop = document.documentElement.scrollTop = location.top
    } else {
      document.body.scrollTop = document.documentElement.scrollTop = 0
    }
  }
})

/* Vue.directive('link',function (el, binding, vnode) {
 el.addEventListener('click',function(){
 if(binding.value){
 router.push(binding.value)
 }else{
 router.push({name:binding.expression})
 }
 })
 }) */

export {historyList}

export default router
