import _ from 'underscore'
import conf from 'assets/js/conf.js'
import {Toast} from 'mint-ui'

const Rxports = {
  style: function (obj) {
    let string = ''
    _.each(obj, function (value, key) {
      string += key + ':' + value + ';'
    })
    return string
  },
  detect: function () {
    let ua = navigator.userAgent
    let self = this
    this.os = {}
    let funcs = [

      function () { // wechat
        var wechat = ua.match(/(MicroMessenger)\/([\d\.]+)/i)
        if (wechat) { // wechat
          this.os.wechat = {
            version: wechat[2].replace(/_/g, '.')
          }
        }
        return false
      },
      function () { // android
        var android = ua.match(/(Android);?[\s\/]+([\d.]+)?/)
        if (android) {
          this.os.android = true
          this.os.version = android[2]

          this.os.isBadAndroid = !(/Chrome\/\d/.test(window.navigator.appVersion))
        }
        return this.os.android === true
      },
      function () { // ios
        var iphone = ua.match(/(iPhone\sOS)\s([\d_]+)/)
        if (iphone) { // iphone
          this.os.ios = this.os.iphone = true
          this.os.version = iphone[2].replace(/_/g, '.')
        } else {
          var ipad = ua.match(/(iPad).*OS\s([\d_]+)/)
          if (ipad) { // ipad
            this.os.ios = this.os.ipad = true
            this.os.version = ipad[2].replace(/_/g, '.')
          }
        }
        return this.os.ios === true
      }
    ]
    ;[].every.call(funcs, function (func) {
      return !func.call(self)
    })
    // 单独判断plus
    let plus = ua.match(/Html5Plus/i)
    if (plus) this.os.plus = true
  },
  toast: function (option) {
    if (typeof option === 'string') {
      let msg = option
      option = {
        message: msg,
        position: 'bottom'
      }
    }
    Toast(option)
  },
  image: function (uri, options) {
    var url = conf.imageHost + uri
    if (_.isEmpty(options)) {
      return url
    }
    if (typeof options.ext === 'undefined') {
      var urlArray = url.split('.')
      options.ext = urlArray[urlArray.length - 1]
    }
    url += '@'
    var optionsStr = []
    for (var i in options) {
      if (i === 'width') {
        optionsStr.push(options[i] + 'w')
      } else if (i === 'height') {
        optionsStr.push(options[i] + 'h')
      } else if (i === 'type') {
        if (options[i] === 'cut') {
          optionsStr.push('1c')
          optionsStr.push('1e')
        }
      } else if (i === 'force') {
        if (options[i] === false) {
          optionsStr.push('1l')
        }
      }
    }
    if (optionsStr.length > 0) {
      url += optionsStr.join('_')
    }
    url += '.' + options.ext
    return url
  },
  /**
   * 拷贝自vue核心库，触发一个vm实例的生命周期钩子
   * @param {Object} vm vue实例
   * @param {String} hook 钩子名称
   */
  callHook: function (vm, hook) {
    const handlers = vm.$options[hook]
    if (handlers) {
      for (let i = 0, j = handlers.length; i < j; i++) {
        handlers[i].call(vm)
      }
    }
    vm.$emit('hook:' + hook)
  },
  /**
   * 清除html中的标签，返回其中的纯文本字符串
   * @param {String} html 要处理的字符串
   * @return {String} 处理后的纯文本字符串
   */
  strip: function (html) {
    let regex = /(<([^>]+)>)/ig
    let text = html.replace(regex, '')
    return text || ''
  },
  setItem: function (name, value, expires) {
    var obj = {
      'app': conf.appname,
      'value': value,
      'expires': expires ? Math.floor(Date.now() / 1000) + expires : 0
    }
    window.plus.storage.setItem(name + '-' + conf.appname, JSON.stringify(obj))
  },
  getItem: function (name) {
    var obj = window.plus.storage.getItem(name + '-' + conf.appname)
    if (obj !== null) {
      obj = JSON.parse(obj)
      if (obj.expires === 0 || obj.expires > Date.now() / 1000) {
        return obj.value
      }
    }
    return null
  },
  removeItem: function (name) {
    window.plus.storage.removeItem(name + '-' + conf.appname)
  },
  isEqual: function (obj1, obj2, strict) {
    var _ = require('underscore')
    if (_.isEqual(obj1, obj2)) {
      return true
    }
    if (strict) {
      return false
    } else {
      if (_.isObject(obj1) && _.isObject(obj2)) {
        for (var i in obj1) {
          if (_.isNumber(obj1[i]) || _.isNumber(obj2[i])) {
            if (obj1[i] !== obj2[i]) {
              return false
            }
          } else {
            if (!this.isEqual(obj1[i], obj2[i])) {
              return false
            }
          }
        }
      } else {
        if (String(obj1) !== String(obj2)) {
          return false
        }
      }

      return true
    }
  }
}
// 判断浏览器环境
Rxports.detect()

module.exports = Rxports
